/**
 * 设计原则：
 *
 * 1.　trigger的触发事件监听不在device.on()上完成，因为设备可能会上线或下线，甚至会重启，导致难以管理。
 *　　　所以，直接在eventBus上订阅propertiesChanged事件，然后检查条件是否满足。［要轮询，比较费时间］
 *　　　
 * 2. action在执行的时候再去关联设备，如果没有设备关联，则取消执行。
 *
 * 3. 除了timer可自检查的事件，目前己知的事件源有propertiesChanged 和　events。
 *
 */
const util = require('util');
const fs = require('fs');
const path = require('path');
const jobmProxy = require('../dbus/dbus-proxy/JobManagerProxy.js');
const Scene = require('./scene.js');
const log = require('../../utils/log');
const constant = require("../../utils/constant");
const { PROJECT_PATH } = require('../../const');
const configurationPath = path.join(constant.FOTA_IFTTT, 'configuration.json');

const TAG = "IFTTT::ifttt";

var IFTTT = function (scheduler, eventBus, statemachine) {
    this.scenes =[]; // parse成功之后的job信息
    this.ifttts = []; // ifttt的job信息
    this.scheduler = scheduler;
    this.eventBus = eventBus;
    this.statemachine = statemachine;
};

IFTTT.ACTION_ADD_SCENE = 10201; // 增加场景
IFTTT.ACTION_DELETE_SCENE = 10204; // 删除场景
IFTTT.COMMAND_DELETE_JOB = 10211; //冗余场景删除

IFTTT.prototype.refreshIfttt = function(info) {
    var refreshed = false;
    if (IFTTT.ACTION_ADD_SCENE === info.ackCode) {
        this.ifttts.push(info);
        refreshed = true;
    } else if (IFTTT.ACTION_DELETE_SCENE === info.ackCode) {
        for (var i = 0; i < this.ifttts.length; ++i) {
            var it = this.ifttts[i];
            if (it.jobId === info.jobId) {
                this.ifttts.splice(i, 1);
                refreshed = true;
                break;
            }
        }
    } else {
        log.i("[ERROR] onIftttEvent can not deal with ackCode : " + info.ackCode);
    }

    log.i('IFTTT refreshIfttt :', refreshed);
    if (refreshed) {
        this.saveConfig();
    }
};

IFTTT.prototype.loadConfig = function() {
    // log.i('[IFTTT::loadConfig] file path :', configurationPath);
    var self = this;
    if (fs.existsSync(configurationPath)) {
        fs.readFile(configurationPath, 'utf-8', function(err, data) {
            if (err) {
                log.i('IFTTT loadConfig failed for :', err.message);
                return;
            }

            try {
                self.ifttts = JSON.parse(data);
                self.ifttts.forEach(function(scene) {
                    self.addScene(scene);
                });
            } catch (e) {
                log.i('IFTTT parse ${configurationPath} failed:', e.message);
            }
        });
    }
};

IFTTT.prototype.saveConfig = function() {
    fs.writeFileSync(configurationPath, JSON.stringify(this.ifttts));
};

IFTTT.prototype.enableScene = function(sceneId) {
    this.scenes.forEach(function(scene) {
        if (sceneId === scene.id) {
            scene.enabled = true;
        }
    });
};

IFTTT.prototype.disableScene = function(sceneId) {
    this.scenes.forEach(function(scene) {
        if (sceneId === scene.id) {
            scene.enabled = false;
        }
    });
};

IFTTT.prototype.addScene = function(info) {
    var filePath = info.path || undefined;
    var isLegal = false;
    var self = this;
    fs.readFile(path.join(filePath, constant.jobIndex), {encoding: 'utf-8'}, function(err, data) {
        if (!err) {
            try {
                var sceneObj = JSON.parse(data);
                var scene = new Scene(self.scheduler, self.eventBus, self.statemachine);

                scene.jobInfo = info;

                isLegal = scene.parseScene(sceneObj);
                if (isLegal) {
                    log.i("parse scene successed.");
                    self.scenes.push(scene);
                } else {
                    log.i(util.format("[WARNING] parse '%s' failed.", filePath));
                }
            } catch (e) {
                log.i(util.format("[ERROR] parse ifttt rule '%s' failed: %s", filePath, e.message));
                log.i(e.stack);
                isLegal = false;
            }
        } else {
            log.i(util.format("[ERROR] read file '%s' failed: %s", filePath, err));
            isLegal = false;
        }
        var response = {};
        response.category = 0;
        response.ackCode = info.ackCode;
        response.jobId = info.jobId;
        response.executor = info.executor;
        var tag = '[IFTTT_Deploy_Result]';
        response.msg = {};
        if (isLegal) {
            response.msg.code = 0;
            response.msg.detail = 'deploy success for jobId: ' + info.jobId;
            response.jobStatus = 2;
            tag += "[Succeeded]";

        } else {
            response.msg.code = 101700;
            response.msg.detail = 'IFTTT_ERROR_CloudDateError';
            log.e(TAG, 'deploy failed for jobId: ' + info.jobId);
            response.jobStatus = 4;
            tag += "[Failed]";
        }
        response.ackId = info.ackId;
        response.sceneId = info.sceneId;
        log.m(tag, '[' +info.sceneId + ']','result = ', response.result);
        console.log("IFTTT sendJobStatus =" + JSON.stringify(response));
        // jobmProxy.sendJobStatus(response);
    });
};

IFTTT.prototype.deleteScene = function(info) {
    log.i('[IFTTT::deleteScene] for', JSON.stringify(info));
    var isSuccess = false;
    var found = false;
    var response = {};
    response.category = 0;
    response.ackCode = info.ackCode;
    response.jobId = info.jobId;
    response.executor = info.executor;
    response.jobStatus = 0;
    response.ackId = info.ackId;
    response.sceneId = info.sceneId;
    var sceneId = info.sceneId;
    var tag = '[IFTTT_RESET_Result][Succeeded]';
    response.msg = {};
    response.msg.code = 0;
    for (var i = 0; i < this.scenes.length; ++i) {
        if (this.scenes[i].jobInfo.jobId === info.jobId) {
            isSuccess = this.scenes[i].stop();
            if (isSuccess) {
                this.scenes.splice(i, 1);
                found = true;
                response.msg.detail = 'delete success for jobId:' + info.jobId;
            }
            break;
        }
    }
    if(!found) {
        response.msg.detail = 'delete success for not existed jobId:' + info.jobId;
    }
    // jobmProxy.sendJobStatus(response);
    log.m(tag, '[' + sceneId + ']', 'result = ', response.msg);
    console.log("IFTTT deleteScene sendJobStatus =" + JSON.stringify(response));
};

IFTTT.prototype.deleteAllScene = function () {
    var isSuccess = false;
    for (var i = this.scenes.length - 1; i >= 0; i--) {
            isSuccess = this.scenes[i].stop();
            if (isSuccess) {
                this.scenes.splice(i, 1);
            }
            this.ifttts.splice(i, 1);
    }
    this.saveConfig();
    log.i('[IFTTT::deleteAllScene] end');
};

module.exports = IFTTT;
